package java.security;

import java.io.ObjectInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.Serializable;

public final class SignedObject implements Serializable
{
    private static final long serialVersionUID = 720502720485447167L;
    private byte[] content;
    private byte[] signature;
    private String thealgorithm;
    
    public SignedObject(Serializable s, PrivateKey privateKey, Signature signature) throws IOException, InvalidKeyException, SignatureException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(out);
        objectOutputStream.writeObject(s);
        objectOutputStream.flush();
        objectOutputStream.close();
        this.content = out.toByteArray();
        out.close();
        this.sign(privateKey, signature);
    }
    
    public Object getObject() throws IOException, ClassNotFoundException {
        ByteArrayInputStream in = new ByteArrayInputStream(this.content);
        ObjectInputStream objectInputStream = new ObjectInputStream(in);
        Object object = objectInputStream.readObject();
        in.close();
        objectInputStream.close();
        return object;
    }
    
    public byte[] getSignature() {
        return (byte[])this.signature.clone();
    }
    
    public String getAlgorithm() {
        return this.thealgorithm;
    }
    
    public boolean verify(PublicKey publicKey, Signature signature) throws InvalidKeyException, SignatureException {
        signature.initVerify(publicKey);
        signature.update((byte[])this.content.clone());
        return signature.verify((byte[])this.signature.clone());
    }
    
    private void sign(PrivateKey privateKey, Signature signature) throws InvalidKeyException, SignatureException {
        signature.initSign(privateKey);
        signature.update((byte[])this.content.clone());
        this.signature = (byte[])signature.sign().clone();
        this.thealgorithm = signature.getAlgorithm();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        this.content = (byte[])((byte[])(byte[])fields.get("content", null)).clone();
        this.signature = (byte[])((byte[])(byte[])fields.get("signature", null)).clone();
        this.thealgorithm = (String)fields.get("thealgorithm", null);
    }
}
